import { serve as serveHttp } from "./deps/http.ts";
import {
	compressResponse,
	setHeaders,
	orElse,
	filter,
	catching,
	log,
} from "./deps/shirt.ts";

import { frontend } from "../build/frontend.ts";
import { style } from "../build/style.ts";
import { serve } from "./api.ts";
import { load as loadConfiguration } from "./config.ts";

export const notFound = (_req: Request) => {
	return new Response("Not found", { status: 404 });
}

const internalServerError = (_req: Request, e: unknown) => {
	console.error(e);
	return new Response("Something went wrong", { status: 500 });
}

const apiHandler = filter(
	(req) => new URL(req.url).pathname.split("/")[1] === "api",
	setHeaders([["content-type", "application/json"]], serve),
);

const root = (req: Request) => {
	if (!req.url.endsWith("/")) {
		return null;
	}

	return new Response(`
<!DOCTYPE html>
<html>
<head>
	<script type="module">
	${frontend}
	</script>
	<style>
	${style}
	</style>
	<title>Statue</title>
</head>
<body>
	<noscript>
		You need to enable JavaScript to view this page.
	</noscript>
</body>
</html>
`, {
	headers: { "content-type": "text/html" },
});
};

const listener =
	log(compressResponse(catching(orElse(
		apiHandler,
		orElse(root, notFound),
	), internalServerError)));

const config = await loadConfiguration();
const port = config.port ?? 8888;
const host = config.hostname ?? "localhost";
const server = serveHttp(listener, {
	hostname: host,
	port,
});

await server;
