import { makeClient, assert, ClientResponse, isHttpError, isValidationError } from "../common/deps/yaypi.ts";
import { swel } from "./deps/swel.ts";

import { api, StatusResponseV1 } from "../common/api.ts";

const client = (() => {
	const l = window.location;
	const url = `${l.protocol}//${l.host}/api`;
	return makeClient(api, url);
})();

const ServiceInfo = (
	name: string,
	getStatus: () => Promise<ClientResponse<StatusResponseV1>>,
	publicUrl?: string,
) => {
	const nameEl = swel("div", { className: "name" }, name);
	const status = swel("div", { className: "status" }, "loading");
	const go = publicUrl ? swel("a", {
		className: "go",
		href: publicUrl,
		target: "_blank",
		on: { mouseenter: (e) => e.preventDefault() },
	}, "Go") : null;

	const button = swel("button", {
		className: "status-button",
		tabIndex: 0,
	}, [nameEl, status]);

	const refresh = async () => {
		button.removeEventListener("click", refresh);
		button.disabled = true;
		status.textContent = "loading";
		button.className = "status-button";

		const st = await getStatus();
		if (isHttpError(st)) {
			status.textContent = "error";
			button.classList.add("error");
		} else if (isValidationError(st)) {
			status.textContent = st.error.toString();
			button.classList.add("error");
		} else if (typeof st === "object") {
			status.textContent = st.error;
			button.classList.add("error");
		} else {
			status.textContent = st;
			button.classList.add(st);
		}

		button.addEventListener("click", refresh);
		button.disabled = false;
	};

	refresh();

	return swel("div", { className: "status-container", }, [button, go]);
}

const main = async () => {
	const resp = await client.v1.configuration.get().then(assert);

	const hostsSection = swel("section", [
		swel("h1", "Hosts"),
		swel(
			"div",
			{ className: "hosts" },
			resp.hosts.map(h => ServiceInfo(
				h,
				() => client.v1.host.getStatus(h),
			))
		),
	]);

	const servicesSection = swel("section", [
		swel("h1", "Services"),
		swel("div", { className: "services", }, resp.services.map(si => ServiceInfo(
			si.name,
			() => client.v1.service.getStatus(si.name),
			si.publicUrl,
		))),
	]);

	document.body.replaceChildren(hostsSection, servicesSection);
};

main();
